/********************************************************************/
/*          sub.c                                                   */
/*            General Use Subroutines                               */
/*                                                                  */
/*          Copyright (C) 1999-2001 NINTENDO Co.,Ltd.               */
/********************************************************************/
//#define CODE32
#include "sio32_multi_load.h"


/*==================================================================*/
/*                      Interrupt Routine                           */
/*==================================================================*/

/*------------------------------------------------------------------*/
/*                      V-Blank Processing                          */
/*------------------------------------------------------------------*/

void VBlankIntr(void)
{
    if (SioFlags & MULTI_SIO_PARENT)                // Start Multi-play Communication (1 frame)
        if (SioState >= 2 && SioState <= 3)
            MultiSioVSync();

    // To make SIO interrupts possible, do not use DMA
    CpuFastArrayCopy(BgBak,  BG_VRAM);              // Set BG Screen
    CpuFastArrayCopy(OamBak, OAM    );              // Set OAM

    *(vu16 *)REG_IME = 0;
    *(vu16 *)INTR_CHECK_BUF |= V_BLANK_INTR_FLAG;   // Set V-Blank Interrupt Check
    *(vu16 *)REG_IME = 1;
}

/*------------------------------------------------------------------*/
/*                      Interrupt Dummy Routine                     */
/*------------------------------------------------------------------*/

void IntrDummy(void)
{
}

/*==================================================================*/
/*                      Subroutine                                  */
/*==================================================================*/

/*------------------------------------------------------------------*/
/*                      Key Read                                    */
/*------------------------------------------------------------------*/

void KeyRead(void)
{
    u16 ReadData = (*(vu16 *)REG_KEYINPUT ^ 0x03ff);
    Trg  = ReadData & (ReadData ^ Cont);            // Trigger Input
    Cont = ReadData;                                // Hold Input
}

/*------------------------------------------------------------------*/
/*                      Set BG Screen                               */
/*------------------------------------------------------------------*/

void BgScSet(u8 *Srcp, u16 *Destp, u8 PlttNo)
{
    while (*Srcp != '\0')
        *Destp++ = (u16 )PlttNo << BG_SC_PLTT_SHIFT | *Srcp++;
}

/*------------------------------------------------------------------*/
/*              Hexadecimal Data -> BG Screen Conversion Routine    */
/*------------------------------------------------------------------*/

void Data2BgSc(u32 SrcData, u16 *Destp, u8 Num, u8 PlttNo)
{
    u32 CharNoTmp;
    u32 PlttNoTmp = PlttNo << BG_SC_PLTT_SHIFT;
    int i;

    SrcData <<= (8-Num)*4;
    for (i=0; i<Num; i++) {
        CharNoTmp = (SrcData >> 28);
        if (CharNoTmp < 0xA)    CharNoTmp += 0x30;
        else                    CharNoTmp += 0x41 - 0xa;
        Destp[i] = CharNoTmp | PlttNoTmp;
        SrcData <<= 4 ;
    }
}


